/* Copyright © 2020 Splunk Inc.
SPLUNK CONFIDENTIAL – Use or disclosure of this material in whole or
in part without a valid written license from Splunk Inc. is PROHIBITED. */
import {
  computeLineAbsPosition,
  getAllBlockItems,
  getAllLineItems,
  getBlockItem
} from "./chunk-X4NAUHLE.js";
import {
  cloneBlockItem,
  cloneLine,
  disconnectLine,
  isFromImageRegistry,
  lodash_exports,
  moveLayoutItem,
  updateBlockItemSize,
  useLayoutShowHide
} from "./chunk-NJ3ZVGVA.js";
import {
  react_default,
  react_exports
} from "./chunk-D2UIVRV7.js";
import {
  BaseLayoutApi_default
} from "./chunk-D6XDCMUA.js";
import {
  absoluteLayoutOptions,
  gridLayoutOptions
} from "./chunk-5YVOBQNO.js";
import {
  dashboard_utils_exports
} from "./chunk-QCW3O7QD.js";
import {
  CLONE_LAYOUT_ITEMS_MSG_PANEL_TOO_SMALL,
  MIN_WIDTH_PX
} from "./chunk-DLFGXT3M.js";
import {
  __async,
  __objRest,
  __spreadProps,
  __spreadValues
} from "./chunk-W3Q7A2F6.js";

// src/apis/AbsoluteLayoutApi.ts
var AbsoluteLayoutApi = class extends BaseLayoutApi_default {
  constructor(_a) {
    var _b = _a, { telemetry } = _b, layout = __objRest(_b, ["telemetry"]);
    super();
    /**
     * Get layout items in order
     * @method
     * @returns {String} ordered item ids
     * @public
     */
    this.getLayoutItemOrder = () => {
      return getAllBlockItems({
        layoutStructure: this.layout.layoutStructureRef.current
      }).map(({ item }) => item);
    };
    /**
     * Adjust layout item order synchronously
     * @method
     * @param {Number} currentOrder
     * @param {Number} newOrder
     * @returns {AbsoluteLayoutStructure} updated layout structure
     * @public
     */
    this.adjustLayoutItemOrderSync = (currentOrder, newOrder, metadata) => {
      this.emitTelemetry({
        pageAction: "adjustLayoutItemOrder",
        metadata
      });
      const blockItems = getAllBlockItems({
        layoutStructure: this.layout.layoutStructureRef.current
      });
      const lineItems = getAllLineItems({
        layoutStructure: this.layout.layoutStructureRef.current
      });
      const updatedBlockItems = moveLayoutItem(
        blockItems,
        currentOrder,
        Math.min(newOrder, blockItems.length)
      );
      return [...lineItems, ...updatedBlockItems];
    };
    /**
     * Adjust layout item order
     * @method
     * @param {Number} currentOrder
     * @param {Number} newOrder
     * @returns {Promise<AbsoluteLayoutStructure>} updated layout structure
     * @public
     */
    this.adjustLayoutItemOrder = (currentOrder, newOrder, metadata) => __async(this, null, function* () {
      return this.adjustLayoutItemOrderSync(currentOrder, newOrder, metadata);
    });
    /**
     * Add a new layout item to the structure synchronously
     * @method
     * @param {Object} options
     * @param {String} options.itemId
     * @param {Object} options.vizContract
     * @param {String} [options.type='block']
     * @param {Object} options.config
     * @returns {AbsoluteLayoutStructure} updated layout structure
     * @public
     */
    this.addLayoutItemSync = ({
      itemId,
      vizContract,
      type = "block",
      config,
      metadata
    }) => {
      this.emitTelemetry({
        pageAction: "addLayoutItem",
        metadata
      });
      return this.addLayoutItemsInternal({
        items: [{ itemId, vizContract, type, config }]
      });
    };
    /**
     * Add a new layout item to the structure
     * @method
     * @param {Object} options
     * @param {String} options.itemId
     * @param {Object} options.vizContract
     * @param {String} [options.type='block']
     * @param {Object} options.config
     * @returns {Promise<AbsoluteLayoutStructure>} updated layout structure
     * @public
     */
    this.addLayoutItem = (_0) => __async(this, [_0], function* ({
      itemId,
      vizContract,
      type = "block",
      config,
      metadata
    }) {
      return this.addLayoutItemSync({ itemId, vizContract, type, config, metadata });
    });
    /**
     * Synchronously generate a new layout structure with one one or more items added
     * @method addLayoutItemsSync
     * @param {Object[]} items
     * @param {Object} [metadata]
     * @returns {AbsoluteLayoutStructure} New layout structure
     * @public
     */
    this.addLayoutItemsSync = (items, metadata) => {
      this.emitTelemetry({
        pageAction: "addLayoutItems",
        metadata
      });
      return this.addLayoutItemsInternal({ items });
    };
    /**
     * Generate a new layout structure with one one or more items added
     * @method addLayoutItems
     * @param {Object[]} items
     * @param {Object} [metadata]
     * @returns {AbsoluteLayoutStructure} New layout structure
     * @public
     */
    this.addLayoutItems = (items, metadata) => __async(this, null, function* () {
      return this.addLayoutItemsSync(items, metadata);
    });
    /**
     * Add multiple new layout items to the structure synchronously
     * @method
     * @param {Object} options
     * @param {Object[]}} options.items
     * @returns {AbsoluteLayoutStructure} updated layout structure
     * @private
     */
    this.addLayoutItemsInternal = ({
      items
    }) => {
      var _a, _b, _c, _d;
      const width = (_b = (_a = this.options) == null ? void 0 : _a.width) != null ? _b : absoluteLayoutOptions.width;
      const height = (_d = (_c = this.options) == null ? void 0 : _c.height) != null ? _d : absoluteLayoutOptions.height;
      return items.reduce(
        (proposedStructure, item) => {
          var _a2;
          const newStructureItem = (0, dashboard_utils_exports.computeNewAbsoluteStructureItem)(__spreadProps(__spreadValues({}, item), {
            config: item.config,
            type: (_a2 = item.type) != null ? _a2 : "block",
            canvasWidth: width,
            canvasHeight: height,
            layoutItems: proposedStructure
          }));
          proposedStructure.push(newStructureItem);
          return proposedStructure;
        },
        [...this.layout.layoutStructureRef.current]
      );
    };
    /**
     * Remove items from layout structure synchronously
     * @method
     * @param {String[]} [itemIds = []] IDs of items to be removed
     * @param {Object} [metadata] Optional telemetry metadata
     * @returns {AbsoluteLayoutStructure} Updated layout structure
     * @public
     */
    this.removeLayoutItemsSync = (itemIds = [], metadata = void 0) => {
      this.emitTelemetry({
        pageAction: "removeLayoutItems",
        metadata
      });
      const updatedItems = [];
      this.layout.layoutStructureRef.current.forEach((item) => {
        let iterItem = item;
        if (item.type === "line") {
          let updatedLine = item;
          const absPos = computeLineAbsPosition({
            layoutStructure: this.layout.layoutStructureRef.current,
            position: updatedLine.position
          });
          if ("item" in updatedLine.position.from && itemIds.indexOf(updatedLine.position.from.item) >= 0) {
            updatedLine = disconnectLine({
              line: updatedLine,
              dir: "from",
              absPos: absPos.from
            });
          }
          if ("item" in updatedLine.position.to && itemIds.indexOf(updatedLine.position.to.item) >= 0) {
            updatedLine = disconnectLine({
              line: updatedLine,
              dir: "to",
              absPos: absPos.to
            });
          }
          iterItem = updatedLine;
        }
        if (itemIds.indexOf(iterItem.item) < 0) {
          updatedItems.push(iterItem);
        }
      });
      return updatedItems;
    };
    /**
     * Remove items from layout structure
     * @method
     * @param {String[]} [itemIds = []] IDs of items to be removed
     * @param {Object} [metadata] Optional telemetry metadata
     * @returns {Promise<AbsoluteLayoutStructure>} Updated layout structure
     * @public
     */
    this.removeLayoutItems = (..._0) => __async(this, [..._0], function* (itemIds = [], metadata = void 0) {
      return this.removeLayoutItemsSync(itemIds, metadata);
    });
    /**
     * Copies a layout item position and size and offsets by `2 * GRID_SIZE` synchronously
     * @method
     * @param {Object} config
     * @param {Array} config.from List of original viz ids
     * @param {Array} config.to List of new viz ids
     * @param {Number} [config.offsetMultiplier=1] Number of grids to offset
     * @returns {AbsoluteLayoutStructure} Layout structure of all known items
     * @public
     * @throws {Error} Will error if `from` or `to` is not an array, or `from.length` != `to.length`
     */
    this.cloneLayoutItemsSync = ({
      from,
      to,
      offsetMultiplier = 1,
      metadata
    }) => {
      if (!Array.isArray(from) || !Array.isArray(to) || from.length !== to.length) {
        throw new Error(
          `Cannot clone item, incorrect inputs from ${from}, to: ${to}`
        );
      }
      this.emitTelemetry({
        pageAction: "cloneLayoutItems",
        metadata
      });
      const currentStructure = Object.fromEntries(
        Object.values(this.layout.layoutStructureRef.current).map(
          (item) => [item.item, item]
        )
      );
      const newLayoutItems = [];
      from.forEach((copyItemId, idx) => {
        const item = currentStructure[copyItemId];
        if (!item) {
          return;
        }
        if ((0, dashboard_utils_exports.isBlockItem)(item)) {
          const clonedBlockItem = cloneBlockItem({
            id: to[idx],
            item,
            offsetMultiplier
          });
          newLayoutItems.push(clonedBlockItem);
        } else if (item.type === "line") {
          const linePosition = computeLineAbsPosition({
            layoutStructure: this.layout.layoutStructureRef.current,
            position: item.position
          });
          const clonedLine = cloneLine({
            id: to[idx],
            item: {
              item: copyItemId,
              type: "line",
              position: linePosition
            },
            offsetMultiplier
          });
          newLayoutItems.push(clonedLine);
        }
      });
      return [...this.layout.layoutStructureRef.current, ...newLayoutItems];
    };
    /**
     * Copies a layout item position and size and offsets by 2 * GRID_SIZE
     * @method
     * @param {Object} config
     * @param {Array} config.from List of original viz ids
     * @param {Array} config.to List of new viz ids
     * @param {Number} [config.offsetMultiplier=1] Number of grids to offset
     * @returns {AbsoluteLayoutStructure} Layout structure of all known items
     * @public
     */
    this.cloneLayoutItems = (_0) => __async(this, [_0], function* ({
      from,
      to,
      offsetMultiplier = 1,
      metadata
    }) {
      return this.cloneLayoutItemsSync({ from, to, offsetMultiplier, metadata });
    });
    /**
     * get dashboard canvas dom element
     */
    this.getCanvasDomElement = () => {
      return this.layout.getCanvasDomElement();
    };
    /**
     * return snapshot of that includes inlined background image
     * @method
     * @returns {Object} Updated Layout options with encoded background image
     * @public
     */
    this.snapshot = () => {
      var _a, _c;
      const _b = (_a = this.options) != null ? _a : {}, { backgroundImage: imgFromOpts } = _b, opts = __objRest(_b, ["backgroundImage"]);
      const srcFromOptions = (_c = imgFromOpts == null ? void 0 : imgFromOpts.src) != null ? _c : "";
      const srcFromImageRegistry = isFromImageRegistry(srcFromOptions);
      const backgroundImage = imgFromOpts ? {
        backgroundImage: __spreadProps(__spreadValues({}, imgFromOpts), {
          src: srcFromImageRegistry ? this.layout.getBgImageSrc() : srcFromOptions
        })
      } : {};
      return {
        options: __spreadValues(__spreadValues({}, opts != null ? opts : {}), backgroundImage)
      };
    };
    /**
     * Sets dashboard scale
     * @param {Number} scale New scale as floating point number
     */
    this.setScale = (scale) => {
      return this.layout.setScale(scale);
    };
    /**
     * Increases dashboard zoom level for one step
     */
    this.zoomIn = () => {
      return this.layout.zoomIn();
    };
    /**
     * Decreases dashboard zoom level for one step
     */
    this.zoomOut = () => {
      return this.layout.zoomOut();
    };
    /**
     * Adjust dashboard's scale to fit the width of the container
     */
    this.fitToWidth = (maxScale = Infinity) => {
      return this.layout.setFitToWidthAndScrollToTopLeft(maxScale);
    };
    /**
     * @returns {Object} Observable that tracks zoom level changes or null if zoom is unsupported
     */
    this.getZoomLevel = () => {
      return this.layout.getZoomObserver();
    };
    this.layout = layout;
    this.telemetry = telemetry;
    Object.defineProperty(this, "options", {
      get() {
        var _a2;
        return (_a2 = this.layout.optionsRef) == null ? void 0 : _a2.current;
      }
    });
  }
};
var AbsoluteLayoutApi_default = AbsoluteLayoutApi;

// src/apis/GridLayoutApi.ts
var empty = {};
var GridLayoutApi = class extends BaseLayoutApi_default {
  constructor({
    layoutStructureRef,
    options = empty,
    userMessage,
    onItemAdded,
    getCanvasDomElement,
    telemetry
  }) {
    super();
    /**
     * Add a new item to grid layout synchronously
     * @method
     * @param {Object} options
     * @param {String} options.itemId visualization id
     * @returns {Object[]} updated layout structure
     * @public
     */
    this.addLayoutItemSync = ({
      itemId,
      type = "block",
      metadata
    }) => {
      this.emitTelemetry({
        pageAction: "addLayoutItem",
        metadata
      });
      return this.addLayoutItemsInternal({ items: [{ itemId, type }] });
    };
    /**
     * Add a new item to grid layout
     * @method
     * @param {Object} options
     * @param {String} options.itemId visualization id
     * @returns {Object[]} updated layout structure
     * @public
     */
    this.addLayoutItem = ({
      itemId,
      type = "block",
      metadata
    }) => this.addLayoutItemSync({ itemId, type, metadata });
    /**
     * @method addLayoutItemsSync
     * Synchronously generate a new layout structure with one one or more items added
     * @param {Object[]} items
     * @param {Object} [metadata]
     * @returns {GridLayoutStructure} New layout structure
     * @public
     */
    this.addLayoutItemsSync = (items, metadata) => {
      this.emitTelemetry({
        pageAction: "addLayoutItems",
        metadata
      });
      return this.addLayoutItemsInternal({ items });
    };
    /**
     * @method addLayoutItems
     * Generate a new layout structure with one one or more items added
     * @param {Object[]} items
     * @param {Object} [metadata]
     * @returns {GridLayoutStructure} New layout structure
     * @public
     */
    this.addLayoutItems = (items, metadata) => this.addLayoutItemsSync(items, metadata);
    /**
     * Add multiple new items to grid layout
     * @method
     * @param {Object} options
     * @param {Object[]} options.items item data
     * @returns {Object[]} updated layout structure
     * @private
     */
    this.addLayoutItemsInternal = ({
      items
    }) => {
      var _a;
      const width = (_a = this.options.width) != null ? _a : gridLayoutOptions.width;
      const proposedLayoutStructure = items.reduce(
        (structure, { itemId, type }) => {
          const newStructureItem = (0, dashboard_utils_exports.computeNewGridStructureItem)({
            itemId,
            type: type != null ? type : "block",
            canvasWidth: width,
            layoutItems: structure
          });
          structure.push(newStructureItem);
          return structure;
        },
        [...this.layoutStructureRef.current]
      );
      this.onItemAdded();
      return proposedLayoutStructure;
    };
    /**
     * Remove item specified in itemIds list synchronously
     * Note: Only removes one item even though it accepts a list of item ids.
     * @method
     * @param {string[]} item ids to remove.
     * @returns {object[]} updated layout structure
     * @public
     */
    this.removeLayoutItemsSync = (itemIds = [], metadata = void 0) => {
      var _a;
      this.emitTelemetry({
        pageAction: "removeLayoutItems",
        metadata
      });
      const items = this.layoutStructureRef.current;
      let itemToRemove;
      try {
        itemToRemove = getBlockItem({
          layoutStructure: items,
          id: itemIds[0]
        });
      } catch (e) {
        return items;
      }
      const width = (_a = this.options.width) != null ? _a : gridLayoutOptions.width;
      const updatedItems = (0, dashboard_utils_exports.updateRemovedVizNeighbors)({
        itemToRemove,
        items,
        width
      });
      const keys = updatedItems.map((item) => item.item);
      const filteredItems = items.filter(
        (vizItem) => vizItem.item !== itemToRemove.item && keys.indexOf(vizItem.item) < 0
      );
      return [...updatedItems, ...filteredItems];
    };
    /**
     * Remove item specified in itemIds list.
     * Note: Only removes one item even though it accepts a list of item ids.
     * @method
     * @param {string[]} item ids to remove.
     * @returns {object[]} updated layout structure
     * @public
     */
    this.removeLayoutItems = (itemIds = [], metadata = void 0) => this.removeLayoutItemsSync(itemIds, metadata);
    /**
     * There are 3 actors in this cloning algorithm:
     * 1. Clone Ancestor
     * 2. Left-side Descendant
     * 3. Right-side Descendant
     * @method
     * @param {Object} options
     * @param {String[]} options.from List of original viz ids (ancestors). These id's will be re-used to produce left-side descendants.
     * @param {String[]} options.to List of new viz ids. Has the id's of right-side descendants.
     * @returns {Object[]} Layout structure of all known items
     * @public
     */
    this.cloneLayoutItemsSync = ({
      from,
      to,
      metadata
    }) => {
      if (!Array.isArray(from) || !Array.isArray(to) || from.length !== to.length) {
        throw new Error(
          `Cannot clone viz, incorrect inputs from ${from}, to: ${to}`
        );
      }
      this.emitTelemetry({
        pageAction: "cloneLayoutItems",
        metadata
      });
      const items = this.layoutStructureRef.current;
      return from.reduce((accumulator, fromItemId, index) => {
        const ancestor = getBlockItem({
          layoutStructure: items,
          id: fromItemId
        });
        if (ancestor.position.w < MIN_WIDTH_PX * 2) {
          this.userMessage({
            message: CLONE_LAYOUT_ITEMS_MSG_PANEL_TOO_SMALL,
            level: "warning",
            sender: "GridLayoutApi"
          });
          return accumulator;
        }
        const lsDescendant = updateBlockItemSize({
          item: __spreadValues({}, ancestor),
          offset: {
            offsetX: -1 * Math.floor(ancestor.position.w / 2),
            offsetY: 0
          },
          dir: "e"
        });
        const rsDescendant = updateBlockItemSize({
          item: __spreadProps(__spreadValues({}, ancestor), {
            item: to[index]
          }),
          offset: {
            offsetX: Math.ceil(ancestor.position.w / 2),
            offsetY: 0
          },
          dir: "w"
        });
        const accumulatorSansAncestor = (0, lodash_exports.reject)(accumulator, {
          item: fromItemId
        });
        return [...accumulatorSansAncestor, lsDescendant, rsDescendant];
      }, items);
    };
    /**
     * There are 3 actors in this cloning algorithm:
     * 1. Clone Ancestor
     * 2. Left-side Descendant
     * 3. Right-side Descendant
     * @method
     * @param {Object} options
     * @param {String[]} options.from List of original viz ids (ancestors). These id's will be re-used to produce left-side descendants.
     * @param {String[]} options.to List of new viz ids. Has the id's of right-side descendants.
     * @returns {Object[]} Layout structure of all known items
     * @public
     */
    this.cloneLayoutItems = ({
      from,
      to,
      metadata
    }) => this.cloneLayoutItemsSync({ from, to, metadata });
    /**
     * Return snapshot as an empty object for the time being
     * @method
     */
    // eslint-disable-next-line class-methods-use-this
    this.snapshot = () => {
      return {};
    };
    /**
     * Return zoom level as null for now
     * @method
     */
    // eslint-disable-next-line class-methods-use-this
    this.getZoomLevel = () => {
      return null;
    };
    this.layoutStructureRef = layoutStructureRef;
    this.options = options;
    this.userMessage = userMessage;
    this.onItemAdded = onItemAdded;
    this.getCanvasDomElement = getCanvasDomElement;
    this.telemetry = telemetry;
  }
};
var GridLayoutApi_default = GridLayoutApi;

// src/enhancers/withLayoutShowHide.tsx
var EmptyStructure = [];
var withLayoutShowHide = (Layout, { schema, reflowFn }) => {
  const EnhancedLayout = (props) => {
    var _a;
    const layoutStructure = useLayoutShowHide(
      (_a = props.layoutStructure) != null ? _a : EmptyStructure,
      reflowFn
    );
    return /* @__PURE__ */ react_default.createElement(
      Layout,
      __spreadProps(__spreadValues({}, props), {
        layoutStructure,
        rawLayoutStructure: props.layoutStructure
      })
    );
  };
  EnhancedLayout.schema = schema;
  return EnhancedLayout;
};

export {
  AbsoluteLayoutApi_default,
  GridLayoutApi_default,
  withLayoutShowHide
};
//# sourceMappingURL=chunk-NRG3QARU.js.map
