import type { TelemetryAPI, EmittableEvent } from '@splunk/dashboard-telemetry';
import type { AbsoluteLayoutOptions, CloneLayoutItemsArgs, LayoutStructureResult } from '@splunk/dashboard-types';
/**
 * An abstract base for a layout API. The `addLayoutItem`, `cloneLayoutItems`, and
 * `removeLayoutItems` methods are all possibly asynchronous, while the `addLayoutItemSync`,
 * `cloneLayoutItemsSync`, and `removeLayoutItemsSync` methods must all be synchronous.
 *
 * @template S The type which represents the layout structure to be handled
 */
declare abstract class BaseLayoutApi<S = unknown> {
    telemetry?: TelemetryAPI;
    emitTelemetry: (payload: EmittableEvent) => boolean;
    /**
     * add new layout item, possibly asynchronously
     */
    abstract addLayoutItem(...args: unknown[]): LayoutStructureResult<S>;
    /**
     * add new layout item synchronously
     */
    addLayoutItemSync(...args: unknown[]): S;
    /**
     * add one or more new layout items synchronously
     */
    abstract addLayoutItemsSync(items: unknown[], metadata?: Record<string, unknown>): S;
    /**
     * add one or more new layout items
     */
    addLayoutItems(items: unknown[], metadata?: Record<string, unknown>): LayoutStructureResult<S>;
    /**
     * copies item w/h and position with an offset, possibly asynchronously
     */
    abstract cloneLayoutItems({ from, to, }: CloneLayoutItemsArgs): LayoutStructureResult<S>;
    /**
     * copies item w/h and position with an offset synchronously
     */
    cloneLayoutItemsSync({ from, to }: CloneLayoutItemsArgs): S;
    /**
     * remove an item from layout, possibly asynchronously
     */
    abstract removeLayoutItems(itemIds: string[]): LayoutStructureResult<S>;
    /**
     * remove an item from layout synchronously
     */
    removeLayoutItemsSync(itemIds: string[]): S;
    /**
     * get dashboard canvas dom element
     */
    abstract getCanvasDomElement(): HTMLElement;
    /**
     * compute snapshot of this layout
     */
    abstract snapshot(): AbsoluteLayoutOptions;
}
export default BaseLayoutApi;
//# sourceMappingURL=BaseLayoutApi.d.ts.map