import type { BehaviorSubject } from 'rxjs';
import type { MutableRefObject } from 'react';
import type { TelemetryAPI, EmittableEvent } from '@splunk/dashboard-telemetry';
import type { AbsoluteLayoutOptions, AbsoluteLayoutStructure, AddLayoutItem, AddLayoutItemArgs, CloneLayoutItemsArgs } from '@splunk/dashboard-types';
import BaseLayoutApi from '../BaseLayoutApi';
type TelemetryMetadata = EmittableEvent['metadata'];
interface LayoutApiArgs {
    getZoomObserver: () => BehaviorSubject<number>;
    setFitToWidthAndScrollToTopLeft: (maxScale?: number) => void;
    layoutStructureRef: MutableRefObject<AbsoluteLayoutStructure>;
    options?: AbsoluteLayoutOptions;
    optionsRef?: MutableRefObject<AbsoluteLayoutOptions | undefined>;
    getCanvasDomElement: () => HTMLElement;
    setScale: (scale: number) => void;
    zoomIn: () => void;
    zoomOut: () => void;
    telemetry?: TelemetryAPI;
    getBgImageSrc: () => string | undefined;
}
type CloneAbsoluteLayoutItemsArgs = CloneLayoutItemsArgs & {
    offsetMultiplier?: number;
    metadata?: TelemetryMetadata;
};
declare class AbsoluteLayoutApi extends BaseLayoutApi<AbsoluteLayoutStructure> {
    private readonly layout;
    private readonly options;
    constructor({ telemetry, ...layout }: LayoutApiArgs);
    /**
     * Get layout items in order
     * @method
     * @returns {String} ordered item ids
     * @public
     */
    getLayoutItemOrder: () => string[];
    /**
     * Adjust layout item order synchronously
     * @method
     * @param {Number} currentOrder
     * @param {Number} newOrder
     * @returns {AbsoluteLayoutStructure} updated layout structure
     * @public
     */
    adjustLayoutItemOrderSync: (currentOrder: number, newOrder: number, metadata?: EmittableEvent["metadata"]) => AbsoluteLayoutStructure;
    /**
     * Adjust layout item order
     * @method
     * @param {Number} currentOrder
     * @param {Number} newOrder
     * @returns {Promise<AbsoluteLayoutStructure>} updated layout structure
     * @public
     */
    adjustLayoutItemOrder: (currentOrder: number, newOrder: number, metadata?: EmittableEvent["metadata"]) => Promise<AbsoluteLayoutStructure>;
    /**
     * Add a new layout item to the structure synchronously
     * @method
     * @param {Object} options
     * @param {String} options.itemId
     * @param {Object} options.vizContract
     * @param {String} [options.type='block']
     * @param {Object} options.config
     * @returns {AbsoluteLayoutStructure} updated layout structure
     * @public
     */
    addLayoutItemSync: ({ itemId, vizContract, type, config, metadata, }: AddLayoutItemArgs) => AbsoluteLayoutStructure;
    /**
     * Add a new layout item to the structure
     * @method
     * @param {Object} options
     * @param {String} options.itemId
     * @param {Object} options.vizContract
     * @param {String} [options.type='block']
     * @param {Object} options.config
     * @returns {Promise<AbsoluteLayoutStructure>} updated layout structure
     * @public
     */
    addLayoutItem: ({ itemId, vizContract, type, config, metadata, }: AddLayoutItemArgs) => Promise<AbsoluteLayoutStructure>;
    /**
     * Synchronously generate a new layout structure with one one or more items added
     * @method addLayoutItemsSync
     * @param {Object[]} items
     * @param {Object} [metadata]
     * @returns {AbsoluteLayoutStructure} New layout structure
     * @public
     */
    addLayoutItemsSync: (items: AddLayoutItem[], metadata?: TelemetryMetadata) => AbsoluteLayoutStructure;
    /**
     * Generate a new layout structure with one one or more items added
     * @method addLayoutItems
     * @param {Object[]} items
     * @param {Object} [metadata]
     * @returns {AbsoluteLayoutStructure} New layout structure
     * @public
     */
    addLayoutItems: (items: AddLayoutItem[], metadata?: TelemetryMetadata) => Promise<AbsoluteLayoutStructure>;
    /**
     * Add multiple new layout items to the structure synchronously
     * @method
     * @param {Object} options
     * @param {Object[]}} options.items
     * @returns {AbsoluteLayoutStructure} updated layout structure
     * @private
     */
    private readonly addLayoutItemsInternal;
    /**
     * Remove items from layout structure synchronously
     * @method
     * @param {String[]} [itemIds = []] IDs of items to be removed
     * @param {Object} [metadata] Optional telemetry metadata
     * @returns {AbsoluteLayoutStructure} Updated layout structure
     * @public
     */
    removeLayoutItemsSync: (itemIds?: string[], metadata?: TelemetryMetadata) => AbsoluteLayoutStructure;
    /**
     * Remove items from layout structure
     * @method
     * @param {String[]} [itemIds = []] IDs of items to be removed
     * @param {Object} [metadata] Optional telemetry metadata
     * @returns {Promise<AbsoluteLayoutStructure>} Updated layout structure
     * @public
     */
    removeLayoutItems: (itemIds?: string[], metadata?: TelemetryMetadata) => Promise<AbsoluteLayoutStructure>;
    /**
     * Copies a layout item position and size and offsets by `2 * GRID_SIZE` synchronously
     * @method
     * @param {Object} config
     * @param {Array} config.from List of original viz ids
     * @param {Array} config.to List of new viz ids
     * @param {Number} [config.offsetMultiplier=1] Number of grids to offset
     * @returns {AbsoluteLayoutStructure} Layout structure of all known items
     * @public
     * @throws {Error} Will error if `from` or `to` is not an array, or `from.length` != `to.length`
     */
    cloneLayoutItemsSync: ({ from, to, offsetMultiplier, metadata, }: CloneAbsoluteLayoutItemsArgs) => AbsoluteLayoutStructure;
    /**
     * Copies a layout item position and size and offsets by 2 * GRID_SIZE
     * @method
     * @param {Object} config
     * @param {Array} config.from List of original viz ids
     * @param {Array} config.to List of new viz ids
     * @param {Number} [config.offsetMultiplier=1] Number of grids to offset
     * @returns {AbsoluteLayoutStructure} Layout structure of all known items
     * @public
     */
    cloneLayoutItems: ({ from, to, offsetMultiplier, metadata, }: CloneAbsoluteLayoutItemsArgs) => Promise<AbsoluteLayoutStructure>;
    /**
     * get dashboard canvas dom element
     */
    getCanvasDomElement: () => HTMLElement;
    /**
     * return snapshot of that includes inlined background image
     * @method
     * @returns {Object} Updated Layout options with encoded background image
     * @public
     */
    snapshot: () => AbsoluteLayoutOptions;
    /**
     * Sets dashboard scale
     * @param {Number} scale New scale as floating point number
     */
    setScale: (scale: number) => void;
    /**
     * Increases dashboard zoom level for one step
     */
    zoomIn: () => void;
    /**
     * Decreases dashboard zoom level for one step
     */
    zoomOut: () => void;
    /**
     * Adjust dashboard's scale to fit the width of the container
     */
    fitToWidth: (maxScale?: number) => void;
    /**
     * @returns {Object} Observable that tracks zoom level changes or null if zoom is unsupported
     */
    getZoomLevel: LayoutApiArgs['getZoomObserver'];
}
export default AbsoluteLayoutApi;
//# sourceMappingURL=AbsoluteLayoutApi.d.ts.map