import type { MutableRefObject } from 'react';
import type { TelemetryAPI, EmittableEvent } from '@splunk/dashboard-telemetry';
import type { UserMessageArgs } from '@splunk/dashboard-context';
import type { AddLayoutItem, AddLayoutItemArgs, CloneLayoutItemsArgs, GridLayoutOptions, GridLayoutStructure, LayoutItemType } from '@splunk/dashboard-types';
import BaseLayoutApi from '../BaseLayoutApi';
type AddGridLayoutItem = AddLayoutItem<LayoutItemType>;
type AddGridLayoutItemArgs = AddLayoutItemArgs<LayoutItemType>;
type TelemetryMetadata = EmittableEvent['metadata'];
interface LayoutApiArgs {
    layoutStructureRef: MutableRefObject<GridLayoutStructure>;
    options?: GridLayoutOptions;
    userMessage: (args: UserMessageArgs) => void;
    onItemAdded: () => void;
    getCanvasDomElement: () => HTMLElement;
    telemetry?: TelemetryAPI;
}
type CloneGridLayoutItemsArgs = CloneLayoutItemsArgs & {
    metadata?: TelemetryMetadata;
};
declare class GridLayoutApi extends BaseLayoutApi<GridLayoutStructure> {
    private readonly layoutStructureRef;
    private readonly options;
    private readonly userMessage;
    private readonly onItemAdded;
    getCanvasDomElement: LayoutApiArgs['getCanvasDomElement'];
    constructor({ layoutStructureRef, options, userMessage, onItemAdded, getCanvasDomElement, telemetry, }: LayoutApiArgs);
    /**
     * Add a new item to grid layout synchronously
     * @method
     * @param {Object} options
     * @param {String} options.itemId visualization id
     * @returns {Object[]} updated layout structure
     * @public
     */
    addLayoutItemSync: ({ itemId, type, metadata, }: AddGridLayoutItemArgs) => GridLayoutStructure;
    /**
     * Add a new item to grid layout
     * @method
     * @param {Object} options
     * @param {String} options.itemId visualization id
     * @returns {Object[]} updated layout structure
     * @public
     */
    addLayoutItem: ({ itemId, type, metadata, }: AddGridLayoutItemArgs) => GridLayoutStructure;
    /**
     * @method addLayoutItemsSync
     * Synchronously generate a new layout structure with one one or more items added
     * @param {Object[]} items
     * @param {Object} [metadata]
     * @returns {GridLayoutStructure} New layout structure
     * @public
     */
    addLayoutItemsSync: (items: AddGridLayoutItem[], metadata?: TelemetryMetadata) => GridLayoutStructure;
    /**
     * @method addLayoutItems
     * Generate a new layout structure with one one or more items added
     * @param {Object[]} items
     * @param {Object} [metadata]
     * @returns {GridLayoutStructure} New layout structure
     * @public
     */
    addLayoutItems: (items: AddGridLayoutItem[], metadata?: TelemetryMetadata) => GridLayoutStructure;
    /**
     * Add multiple new items to grid layout
     * @method
     * @param {Object} options
     * @param {Object[]} options.items item data
     * @returns {Object[]} updated layout structure
     * @private
     */
    private readonly addLayoutItemsInternal;
    /**
     * Remove item specified in itemIds list synchronously
     * Note: Only removes one item even though it accepts a list of item ids.
     * @method
     * @param {string[]} item ids to remove.
     * @returns {object[]} updated layout structure
     * @public
     */
    removeLayoutItemsSync: (itemIds?: string[], metadata?: TelemetryMetadata) => GridLayoutStructure;
    /**
     * Remove item specified in itemIds list.
     * Note: Only removes one item even though it accepts a list of item ids.
     * @method
     * @param {string[]} item ids to remove.
     * @returns {object[]} updated layout structure
     * @public
     */
    removeLayoutItems: (itemIds?: string[], metadata?: TelemetryMetadata) => GridLayoutStructure;
    /**
     * There are 3 actors in this cloning algorithm:
     * 1. Clone Ancestor
     * 2. Left-side Descendant
     * 3. Right-side Descendant
     * @method
     * @param {Object} options
     * @param {String[]} options.from List of original viz ids (ancestors). These id's will be re-used to produce left-side descendants.
     * @param {String[]} options.to List of new viz ids. Has the id's of right-side descendants.
     * @returns {Object[]} Layout structure of all known items
     * @public
     */
    cloneLayoutItemsSync: ({ from, to, metadata, }: CloneGridLayoutItemsArgs) => GridLayoutStructure;
    /**
     * There are 3 actors in this cloning algorithm:
     * 1. Clone Ancestor
     * 2. Left-side Descendant
     * 3. Right-side Descendant
     * @method
     * @param {Object} options
     * @param {String[]} options.from List of original viz ids (ancestors). These id's will be re-used to produce left-side descendants.
     * @param {String[]} options.to List of new viz ids. Has the id's of right-side descendants.
     * @returns {Object[]} Layout structure of all known items
     * @public
     */
    cloneLayoutItems: ({ from, to, metadata, }: CloneGridLayoutItemsArgs) => GridLayoutStructure;
    /**
     * Return snapshot as an empty object for the time being
     * @method
     */
    snapshot: () => Record<string, never>;
    /**
     * Return zoom level as null for now
     * @method
     */
    getZoomLevel: () => null;
}
export default GridLayoutApi;
//# sourceMappingURL=GridLayoutApi.d.ts.map